"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NLogger = void 0;
const common_1 = require("@nestjs/common");
const util_1 = require("util");
const logger_constants_1 = require("./logger.constants");
const logger_definition_1 = require("./logger.definition");
const logger_service_1 = require("./logger.service");
let NLogger = class NLogger extends common_1.ConsoleLogger {
    constructor(opts, loggerService) {
        super();
        this.opts = opts;
        this.loggerService = loggerService;
    }
    log(message, ...optionalParams) {
        var _a;
        const { messages, context } = super['getContextAndMessagesToPrint']([
            message,
            ...optionalParams,
        ]);
        this.pushToDb('log', context, messages);
        (_a = this.loggerService.winston) === null || _a === void 0 ? void 0 : _a.log(message, ...optionalParams);
        super.log(message, ...optionalParams);
    }
    error(message, ...optionalParams) {
        var _a, _b;
        const { messages, context, stack } = super['getContextAndStackAndMessagesToPrint']([
            message,
            ...optionalParams,
        ]);
        this.pushToDb('error', context, messages, stack);
        if (message instanceof Error) {
            (_a = this.loggerService.winston) === null || _a === void 0 ? void 0 : _a.error(message.message, { stack: message.stack }, ...optionalParams);
        }
        else {
            (_b = this.loggerService.winston) === null || _b === void 0 ? void 0 : _b.error(message, ...optionalParams);
        }
        super.error(message, ...optionalParams);
    }
    warn(message, ...optionalParams) {
        var _a;
        const { messages, context } = super['getContextAndMessagesToPrint']([
            message,
            ...optionalParams,
        ]);
        this.pushToDb('warn', context, messages);
        (_a = this.loggerService.winston) === null || _a === void 0 ? void 0 : _a.warn(message, ...optionalParams);
        super.warn(message, ...optionalParams);
    }
    debug(message, ...optionalParams) {
        var _a;
        const { messages, context } = super['getContextAndMessagesToPrint']([
            message,
            ...optionalParams,
        ]);
        this.pushToDb('debug', context, messages);
        (_a = this.loggerService.winston) === null || _a === void 0 ? void 0 : _a.debug(message, ...optionalParams);
        super.debug(message, ...optionalParams);
    }
    verbose(message, ...optionalParams) {
        var _a;
        const { messages, context } = super['getContextAndMessagesToPrint']([
            message,
            ...optionalParams,
        ]);
        this.pushToDb('verbose', context, messages);
        (_a = this.loggerService.winston) === null || _a === void 0 ? void 0 : _a.verbose(message, ...optionalParams);
        super.verbose(message, ...optionalParams);
    }
    pushToDb(type, context, messages, stack) {
        var _a;
        const blacklist = (_a = this.opts.skipContexts) !== null && _a !== void 0 ? _a : logger_constants_1.DEFAULT_SKIP_CONTEXTS;
        const ctx = this.getContext(context);
        if (blacklist.includes(ctx))
            return;
        const data = {
            type,
            context: this.getContext(context),
            data: this.getMessages(messages, stack),
        };
        this.loggerService.pushDatabaseItem({ type: 'log', data });
    }
    getMessages(messages, stack) {
        const ins = (obj) => (0, util_1.inspect)(obj, false, 8);
        if (!messages && !stack)
            return null;
        if (!stack)
            return ins(messages);
        if (Array.isArray(messages)) {
            return ins([...messages, stack]);
        }
        else {
            return ins([messages, stack]);
        }
    }
    getContext(ctx) {
        if (!ctx)
            return null;
        if (typeof ctx === 'string')
            return ctx;
        return (0, util_1.inspect)(ctx);
    }
    formatPid(pid) {
        if (!this.opts.logPrefix) {
            return super.formatPid(pid);
        }
        const prefix = `[${this.opts.logPrefix.slice(0, 12)}]`;
        return prefix.padEnd(14, ' ') + ' - ';
    }
};
exports.NLogger = NLogger;
exports.NLogger = NLogger = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, common_1.Inject)(logger_definition_1.MODULE_OPTIONS_TOKEN)),
    __metadata("design:paramtypes", [Object, logger_service_1.NLoggerService])
], NLogger);
//# sourceMappingURL=logger.logger.js.map