"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NLoggerMiddleware = void 0;
const common_1 = require("@nestjs/common");
const nestjs_cls_1 = require("nestjs-cls");
const request_ip_1 = require("request-ip");
const util_1 = require("util");
const uuid_1 = require("uuid");
const logger_constants_1 = require("./logger.constants");
const logger_definition_1 = require("./logger.definition");
const logger_service_1 = require("./logger.service");
let NLoggerMiddleware = class NLoggerMiddleware {
    constructor(options, cls, service) {
        this.options = options;
        this.cls = cls;
        this.service = service;
    }
    use(req, res, next) {
        return __awaiter(this, void 0, void 0, function* () {
            const id = this.beforeRequest(req, res);
            res.on('finish', () => {
                this.afterRequest(req, res, id);
            });
            next();
        });
    }
    beforeRequest(req, res) {
        const id = (0, uuid_1.v4)();
        this.cls.set(logger_constants_1.REQUEST_ID_KEY, id);
        this.any(req).timestamp = Date.now();
        res.send = this.responseBodyInterceptor(res, res.send);
        return id;
    }
    afterRequest(req, res, id) {
        var _a;
        const data = this.createRequestLog(id, req, res);
        if (data) {
            this.service.pushDatabaseItem({ type: 'request', data });
            const log = this.createConsoleLog(data);
            if (log)
                console.log(log);
            (_a = this.service.winston) === null || _a === void 0 ? void 0 : _a.log(this.getRequestText(data), data);
        }
    }
    createRequestLog(id, req, res) {
        var _a, _b, _c, _d;
        try {
            const responseBody = this.isErrorResponse(res)
                ? this.safeJsonParse(this.any(res).contentBody)
                : null;
            const log = {
                id,
                method: req.method,
                path: req.path,
                params: req.params,
                query: this.hideProperties(req.query),
                body: this.hideProperties(req.body),
                headers: this.getHeaders(req),
                status: res.statusCode,
                error: res.statusCode >= 400 && res.statusCode <= 599,
                ip: (0, request_ip_1.getClientIp)(req),
                responseBody: responseBody,
                timestamp: new Date(this.any(req).timestamp),
                executionTime: Date.now() - this.any(req).timestamp,
                userId: (_b = (_a = this.options).userIdExtractor) === null || _b === void 0 ? void 0 : _b.call(_a, req, this.cls),
                customData: (_d = (_c = this.options).customDataExtractor) === null || _d === void 0 ? void 0 : _d.call(_c, req, this.cls),
            };
            return log;
        }
        catch (e) {
            console.error('Failed to create request log:', e);
            return null;
        }
    }
    createConsoleLog(data) {
        var _a;
        try {
            const details = {
                params: data.params,
                query: data.query,
                body: data.body,
                headers: data.headers,
                ip: data.ip,
                responseBody: data.responseBody,
                userId: data.userId,
                timestamp: (_a = data.timestamp) === null || _a === void 0 ? void 0 : _a.toString(),
                executionTime: `${data.executionTime}ms`,
                customData: data.customData,
            };
            this.clearEmptyValues(details);
            const detailsString = (0, util_1.inspect)(details, false, 8);
            const color = this.withColor(this.matchColorForStatus(data.status));
            const header = color(`[${data.method} ${data.path}] [${data.status}] [${data.id}]`);
            return `${header} ${detailsString}`;
        }
        catch (e) {
            console.error('Failed to create console log:', e);
            return null;
        }
    }
    safeJsonParse(err) {
        if (!err)
            return null;
        try {
            if (typeof err === 'object')
                return Object.assign({}, err);
            if (typeof err === 'string')
                return JSON.parse(err);
            return { raw: (0, util_1.inspect)(err, false, 5) };
        }
        catch (e) {
            return { raw: (0, util_1.inspect)(e, false, 5) };
        }
    }
    isErrorResponse(res) {
        return res.statusCode >= 400 && res.statusCode <= 599;
    }
    getHeaders(req) {
        var _a;
        const headers = (_a = this.options.headers) !== null && _a !== void 0 ? _a : logger_constants_1.DEFAULT_HEADERS;
        const obj = headers.reduce((acc, next) => (Object.assign(Object.assign({}, acc), { [next]: req.header(next) })), {});
        return this.clearEmptyValues(obj);
    }
    responseBodyInterceptor(res, send) {
        return (content) => {
            this.any(res).contentBody = content;
            res.send = send;
            return res.send(content);
        };
    }
    clearEmptyValues(obj) {
        for (const [key, value] of Object.entries(obj)) {
            if (!value)
                delete obj[key];
            if (typeof value === 'object' && value !== null && !Object.keys(value).length) {
                delete obj[key];
            }
        }
        return obj;
    }
    any(value) {
        return value;
    }
    getRequestText(obj) {
        var _a;
        const color = this.withColor(this.matchColorForStatus(obj.status));
        return color(`${(_a = obj.method) === null || _a === void 0 ? void 0 : _a.toUpperCase()} ${obj.path} [${obj.status}]`);
    }
    withColor(color) {
        const COLORS = {
            success: '\x1b[32m',
            error: '\x1b[31m',
            warning: '\x1b[33m',
            default: '\x1b[0m',
        };
        return (text) => `${COLORS[color]}${text !== null && text !== void 0 ? text : ''}${COLORS.default}`;
    }
    matchColorForStatus(status) {
        if (status >= 100 && status < 400) {
            return 'success';
        }
        if (status >= 400 && status < 500) {
            return 'warning';
        }
        if (status >= 500 && status < 600) {
            return 'error';
        }
        return 'default';
    }
    hideProperties(obj) {
        var _a;
        if (!obj || typeof obj !== 'object')
            return obj;
        for (const hidden of (_a = this.options.hiddenProperties) !== null && _a !== void 0 ? _a : []) {
            if (obj[hidden]) {
                obj[hidden] = logger_constants_1.REPLACE_SECRET_STRING;
            }
        }
        return obj;
    }
};
exports.NLoggerMiddleware = NLoggerMiddleware;
exports.NLoggerMiddleware = NLoggerMiddleware = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, common_1.Inject)(logger_definition_1.MODULE_OPTIONS_TOKEN)),
    __metadata("design:paramtypes", [Object, nestjs_cls_1.ClsService,
        logger_service_1.NLoggerService])
], NLoggerMiddleware);
//# sourceMappingURL=logger.middleware.js.map