"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NLoggerService = void 0;
const core_1 = require("@mikro-orm/core");
const postgresql_1 = require("@mikro-orm/postgresql");
const common_1 = require("@nestjs/common");
const nest_winston_1 = require("nest-winston");
const nestjs_cls_1 = require("nestjs-cls");
const node_os_1 = require("node:os");
const winston = require("winston");
const winston_syslog_1 = require("winston-syslog");
const log_entity_1 = require("./entities/log.entity");
const request_entity_1 = require("./entities/request.entity");
const logger_constants_1 = require("./logger.constants");
const logger_definition_1 = require("./logger.definition");
const migrations_1 = require("./migrations");
let NLoggerService = class NLoggerService {
    constructor(options, clsService) {
        this.options = options;
        this.clsService = clsService;
        this.queue = [];
        this.winston = this.createWinstonInstance();
    }
    onApplicationBootstrap() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.orm = yield core_1.MikroORM.init(this.getConnectionConfig(), true);
                const migrator = this.orm.getMigrator();
                yield migrator.getStorage().ensureTable();
                yield migrator.up();
            }
            catch (e) {
                console.error('Failed to connect to db:', e);
            }
            this.interval = setInterval(() => {
                this.insertDatabaseItems();
            }, (_a = this.options.dbConfig.updateInternvalMs) !== null && _a !== void 0 ? _a : 1500);
        });
    }
    onApplicationShutdown() {
        return __awaiter(this, void 0, void 0, function* () {
            clearInterval(this.interval);
            yield this.orm.close(true);
        });
    }
    pushDatabaseItem(item) {
        var _a;
        if (item.type === 'log' && this.clsService.isActive()) {
            item.data.requestId = (_a = this.clsService.get(logger_constants_1.REQUEST_ID_KEY)) !== null && _a !== void 0 ? _a : null;
        }
        this.queue.push(item);
    }
    insertDatabaseItems() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.queue.length)
                return;
            const batch = this.queue;
            this.queue = [];
            try {
                yield this.insertFilteredItems(batch, [request_entity_1.RequestEntity, 'request']);
                yield this.insertFilteredItems(batch, [log_entity_1.LogEntity, 'log']);
            }
            catch (e) {
                console.error('Failed to insert request logs to db:', e);
            }
        });
    }
    insertFilteredItems(batch, [Entity, type]) {
        return __awaiter(this, void 0, void 0, function* () {
            const filtered = batch.filter((el) => el.type === type).map((el) => el.data);
            if (!filtered.length)
                return;
            yield this.orm.em.insertMany(Entity, filtered);
        });
    }
    getConnectionConfig() {
        return (0, postgresql_1.defineConfig)(Object.assign(Object.assign({}, this.options.dbConfig), { schema: 'nlogger', entities: [request_entity_1.RequestEntity, log_entity_1.LogEntity], migrations: {
                tableName: 'migrations',
                migrationsList: migrations_1.migrations.map((migration) => ({ class: migration, name: migration.name })),
                disableForeignKeys: false,
            } }));
    }
    createWinstonInstance() {
        var _a;
        const cfg = this.options.papertrailConfig;
        if (!cfg)
            return null;
        return nest_winston_1.WinstonModule.createLogger({
            transports: [
                new winston_syslog_1.Syslog({
                    host: cfg.host,
                    port: cfg.port,
                    protocol: (_a = cfg.protocol) !== null && _a !== void 0 ? _a : 'tls4',
                    localhost: (0, node_os_1.hostname)(),
                    appName: cfg.systemName,
                    eol: '\n',
                    format: winston.format.combine(winston.format.colorize(), winston.format.simple()),
                }),
            ],
        });
    }
};
exports.NLoggerService = NLoggerService;
exports.NLoggerService = NLoggerService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, common_1.Inject)(logger_definition_1.MODULE_OPTIONS_TOKEN)),
    __metadata("design:paramtypes", [Object, nestjs_cls_1.ClsService])
], NLoggerService);
//# sourceMappingURL=logger.service.js.map